=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2016-05-05 12:18:37 +0000
+++ CMakeLists.txt	2016-08-10 12:23:31 +0000
@@ -3,7 +3,7 @@
 
 project(libertine 
         VERSION 0.99.1
-        LANGUAGES CXX)
+        LANGUAGES C CXX)
 
 set(CMAKE_MODULE_PATH "${CMAKE_SOURCE_DIR}/cmake" "${CMAKE_MODULE_PATH}")
 
@@ -17,11 +17,14 @@
 
 include_directories(${CMAKE_BINARY_DIR} ${CMAKE_SOURCE_DIR})
 
-find_package(Qt5Core  REQUIRED)
-find_package(Qt5Gui   REQUIRED)
-find_package(Qt5Quick REQUIRED)
-find_package(Gettext  REQUIRED)
-find_package(Intltool REQUIRED)
+find_package(Qt5Core      REQUIRED)
+find_package(Qt5Gui       REQUIRED)
+find_package(Qt5Quick     REQUIRED)
+find_package(Qt5Widgets   REQUIRED)
+find_package(Qt5X11Extras REQUIRED)
+find_package(Gettext      REQUIRED)
+find_package(Intltool     REQUIRED)
+find_package(X11          REQUIRED)
 
 pkg_check_modules(GLIB2 REQUIRED glib-2.0)
 include_directories(${GLIB2_INCLUDE_DIRS})
@@ -37,6 +40,7 @@
 add_subdirectory(tools)
 add_subdirectory(liblibertine)
 add_subdirectory(po)
+add_subdirectory(pasted)
 
 include(CTest)
 add_subdirectory(tests)

=== modified file 'debian/control'
--- debian/control	2016-07-21 16:54:50 +0000
+++ debian/control	2016-08-10 12:23:31 +0000
@@ -9,9 +9,12 @@
                dh-translations,
                gobject-introspection,
                intltool,
+               libcontent-hub-dev (>= 0.2),
                libgirepository1.0-dev,
                libglib2.0-dev,
                libgtest-dev,
+               libqt5x11extras5-dev,
+               libx11-dev,
                libpam0g-dev,
                lsb-release,
                python3-apt,

=== modified file 'debian/libertine-tools.install'
--- debian/libertine-tools.install	2016-07-15 18:39:58 +0000
+++ debian/libertine-tools.install	2016-08-10 12:23:31 +0000
@@ -2,6 +2,7 @@
 usr/bin/libertine-session-bridge
 usr/bin/libertine-container-manager
 usr/bin/libertine-xmir
+usr/bin/pasted
 usr/share/bash-completion/completions/libertine-container-manager
 usr/lib/*/libertine/update-puritine-containers
 usr/share/man

=== added directory 'pasted'
=== added file 'pasted/CMakeLists.txt'
--- pasted/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ pasted/CMakeLists.txt	2016-08-10 12:23:31 +0000
@@ -0,0 +1,10 @@
+set(pasted_SRC
+    pasted.cpp
+)
+
+add_executable(pasted ${pasted_SRC})
+
+qt5_use_modules(pasted DBus)
+
+target_link_libraries(pasted Qt5::Core Qt5::Gui Qt5::Widgets Qt5::X11Extras content-hub X11)
+install(TARGETS pasted RUNTIME DESTINATION ${CMAKE_INSTALL_BINDIR})

=== added file 'pasted/pasted.cpp'
--- pasted/pasted.cpp	1970-01-01 00:00:00 +0000
+++ pasted/pasted.cpp	2016-08-10 12:23:31 +0000
@@ -0,0 +1,179 @@
+/**
+ * @file pasted.cpp
+ * @brief Copy & Paste daemon between X apps, other X apps, and native apps
+ */
+/*
+ * Copyright 2016 Canonical Ltd
+ *
+ * Libertine is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License, version 3, as published by the
+ * Free Software Foundation.
+ *
+ * Libertine is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#include "pasted.h"
+
+#include <QDebug>
+
+#include <QX11Info>
+#include <X11/Xlib.h>
+
+Pasted::
+Pasted(int argc, char** argv)
+: QApplication(argc, argv)
+, clipboard_(QApplication::clipboard())
+, content_hub_(cuc::Hub::Client::instance())
+, mimeDataX_(new QMimeData)
+, lastMimeData_(nullptr)
+, rootWindowHasFocus_(false)
+{
+  setApplicationName(QString("pasted"));
+
+  QTimer *timer = new QTimer(this);
+  connect(timer, &QTimer::timeout, this, &Pasted::checkForAppFocus);
+  timer->start(400);
+
+  connect(clipboard_, &QClipboard::dataChanged, this, &Pasted::handleXClipboard);
+}
+
+
+void Pasted::
+handleXClipboard()
+{
+  qDebug() << "Change in X clipboard";
+
+  const QMimeData *xClipboard = clipboard_->mimeData();
+
+  if (!xClipboard || xClipboard->formats().empty())
+  {
+    qDebug() << "Empty xClipboard.  Not setting pasteboard.";
+  }
+  else if (!compareMimeData(xClipboard, lastMimeData_))
+  {
+    qDebug() << "X clipboard data is different";
+    updateLastMimeData(xClipboard);
+ 
+    content_hub_->createPasteSync(*lastMimeData_);
+  }
+}
+
+
+void Pasted::
+handleContentHubPasteboard()
+{
+  const QMimeData *pasteboard = content_hub_->latestPaste();
+
+  if (!compareMimeData(pasteboard, lastMimeData_))
+  {
+    qDebug() << "content-hub pasteboard data is different";
+    updateXMimeData(pasteboard);
+
+    clipboard_->setMimeData(mimeDataX_);
+  }
+}
+
+
+void Pasted::
+checkForAppFocus()
+{
+  Window w;
+  int revert_to;
+
+  XGetInputFocus(QX11Info::display(), &w, &revert_to);
+
+  if (w == None && rootWindowHasFocus_ == true)
+  {
+    qDebug() << "Xmir lost focus";
+    rootWindowHasFocus_ = false;
+  }
+  else if (w && rootWindowHasFocus_ == false)
+  {
+    qDebug() << "Xmir gained focus";
+    rootWindowHasFocus_ = true;
+    handleContentHubPasteboard();
+  }
+}
+
+
+bool Pasted::
+compareMimeData(const QMimeData *a, const QMimeData *b)
+{
+  if (!a || !b)
+  {
+    return false;
+  }
+
+  if (a->formats() != b->formats())
+  {
+    return false;
+  }
+
+  for (const QString &formatName: a->formats())
+  {
+    if (a->data(formatName) != b->data(formatName))
+    {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+
+QMimeData *Pasted::
+copyMimeData(QMimeData *target, const QMimeData *source)
+{
+  if (!source)
+  {
+    qDebug() << "NULL!";
+    return nullptr;
+  }
+
+  for (const QString format : source->formats())
+  {
+    target->setData(format, source->data(format));
+  }
+
+  return target;
+}
+
+
+void Pasted::
+updateLastMimeData(const QMimeData *source)
+{
+  if (lastMimeData_)
+  {
+    delete lastMimeData_;
+  }
+
+  lastMimeData_ = new QMimeData;
+
+  lastMimeData_ = copyMimeData(lastMimeData_, source);
+}
+
+
+void Pasted::
+updateXMimeData(const QMimeData *source)
+{
+  updateLastMimeData(source);
+
+  mimeDataX_ = new QMimeData;
+  mimeDataX_ = copyMimeData(mimeDataX_, source);
+}
+
+
+int
+main(int argc, char* argv[])
+{
+  qSetMessagePattern(QString("%{appname}: %{message}"));
+
+  Pasted pasted(argc, argv);
+
+  pasted.exec();
+}

=== added file 'pasted/pasted.h'
--- pasted/pasted.h	1970-01-01 00:00:00 +0000
+++ pasted/pasted.h	2016-08-10 12:23:31 +0000
@@ -0,0 +1,61 @@
+/**
+ * @file pasted.h
+ * @brief Copy & Paste daemon between X apps, other X apps, and native apps
+ */
+/*
+ * Copyright 2016 Canonical Ltd
+ *
+ * Libertine is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License, version 3, as published by the
+ * Free Software Foundation.
+ *
+ * Libertine is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#ifndef _PASTED_H_
+#define _PASTED_H_
+
+#include <QApplication>
+#include <QDBusInterface>
+#include <QClipboard>
+#include <QMimeData>
+
+#include <com/ubuntu/content/hub.h>
+
+namespace cuc = com::ubuntu::content;
+
+
+class Pasted
+: public QApplication
+{
+  Q_OBJECT
+
+  public:
+    Pasted(int argc, char** argv);
+    virtual ~Pasted() = default;
+
+  private:
+    bool compareMimeData(const QMimeData *a, const QMimeData *b);
+    QMimeData *copyMimeData(QMimeData *target, const QMimeData *source);
+    void updateLastMimeData(const QMimeData *source);
+    void updateXMimeData(const QMimeData *source);
+    void handleContentHubPasteboard();
+
+  private slots:
+    void handleXClipboard();
+    void checkForAppFocus();
+
+  private:
+    QClipboard *clipboard_;
+    cuc::Hub *content_hub_;
+    QMimeData *mimeDataX_;
+    QMimeData *lastMimeData_;
+    bool rootWindowHasFocus_;
+};
+
+#endif /* _PASTED_H_ */

